missiles = {}
missiles.launched = {}
missiles.icbm = love.graphics.newImage("icbm.png")


target = {
	x = love.graphics.getWidth()/2,
	y = love.graphics.getHeight()/2
}

function ripairs(t)
	--same as ipairs, but itterate from last to first
	local function ripairs_it(t,i)
		i=i-1
		local v=t[i]
		if v==nil then return v end
		return i,v
	end
	return ripairs_it, t, #t+1
end



function love.mousepressed(x,y,button)
	--left click to launch a missile
	if button == 1 then
		missiles.add(x,y,target.x,target.y)
	end

	--right click to set a target
	if button == 2 then
		target.x = x
		target.y = y
	end

end


function missiles.add(x,y)

	local ax = x
	local ay = y
	local tx = target.x
	local ty = target.y


	--values for the bezier curve
	local bx,by
	if ax < tx then
		bx = ax + (tx - ax)
	else
		bx = tx+ (ax -tx)
	end

	if ay < ty then
		by = 0
	else
		by = 0
	end

	local trajectory = love.math.newBezierCurve(
		tx,ty,bx,by,ax,ay
	)


	table.insert(missiles.launched, {
		x = ax, -- missile source
		y = ay,

		mx = ax, -- missile projectile
		my = ay,

		tx = tx, -- missile destination
		ty = ty,

		dx = 0, -- missile derivative
		dy = 0,

		t = 0, -- missile time / distance

		trajectory = trajectory, -- the bezier curves
		derivative = trajectory:getDerivative(),

		angle = 0, -- angle of missile sprite
		o = 200, --opacity
		speed = 0.25, -- speed used for "t"
		info = false, -- hud overlay
		weapon = "ICBM" -- weapon type
	})

	--sound.playfx("launch")


end

function love.update(dt)
	for i, missile in ripairs(missiles.launched) do

		missile.t = missile.t+missile.speed*dt


		if missile.t > 1 then
			--missile reaches destination
			table.remove(missiles.launched,i)
		else

			--evaluate missile trajectory
			missile.mx,missile.my = missile.trajectory:evaluate(missile.t)

			--evaluate missile derivative trajectory
			missile.dx,missile.dy = missile.derivative:evaluate(missile.t)

			--find the difference in angle (draw missile sprite using this)
			--missile.angle = math.atan2(missile.mx-missile.dx, missile.my-missile.dy)
			missile.angle = math.atan2 ( missile.dy, missile.dx ) + math.pi / 2 --sprite is rotated wrong
		end
	end
end

function love.draw()
	for i, missile in ipairs(missiles.launched) do


		love.graphics.setColor(0,255,0,200)
		love.graphics.circle("fill",missile.mx,missile.my,10)

		--draw incremental beacons across trajectory
		for i=0,10 do
			local x,y = missile.trajectory:evaluate(i/10)
			local size = 5
			love.graphics.rectangle("fill",x-size/2,y-size/2,size,size)

			local x,y = missile.derivative:evaluate(i/10)
			local size = 5
			love.graphics.rectangle("fill",x-size/2,y-size/2,size,size)
		end

		--the bezier curve (trajectory)
		love.graphics.setColor(
			255,0,0,missile.o
		)
		love.graphics.line(missile.trajectory:render())

		--derivative curve
		love.graphics.line(missile.derivative:render())


		--position of point/time on the bezier curves
		love.graphics.circle("fill",missile.mx,missile.my,5)
		love.graphics.circle("fill",missile.dx,missile.dy,5)


		love.graphics.line(missile.mx,missile.my,missile.dx,missile.dy)


		love.graphics.setColor(
			255,155,0,255
		)
		love.graphics.draw(missiles.icbm, missile.mx,missile.my, missile.angle, 1,1,missiles.icbm:getWidth()/2,missiles.icbm:getHeight()/2)


	end

	love.graphics.setColor(255,255,255,255)
	love.graphics.print("right click to select launch site\nleft click to launch missile",10,10)
end

